// Copyright 1992 by Jon Dart.  All Rights Reserved.

#ifndef _MOVE_ORDERING_H
#define _MOVE_ORDERING_H

#include "board.h"
#include "movegen.h"

class Move_Ordering
{
    // sorts moves according to plausibility, maintains killer list.

    public:	
	
    static void sort_moves( Move moves[], int scores[], int n);
    // sort "n" moves in order of scores.

    static int order_ply0_moves(Board &board, Move moves[], const int num_moves,
            const Move &best_move, int &easy );	    
    // Do initial ordering of moves returned by the move generator.
    // Set "easy" = true if the 1st (pv) move is measurably better than
    //the others.

    static int reorder_ply0_moves( const Board &board, Move moves[], const int num_moves,
		            const Move &best_move);
    // Re-order ply 0 moves after the initial sort

    static int order_captures(Board &board, Move moves[], const int num_moves,
		            const int alpha, const int material, int ply);
    // order moves in the quiescence search

    static int score( Board &board, const Move &move);
    // return a score for an individual move.  The higher the score,
    // the earlier in the search order it should be tried.
	    
private:

};

#endif
