// Copyright 1992 by Jon Dart.  All Rights Reserved.

#ifndef _SCORING_H
#define _SCORING_H

#include "board.h"
#include "bitprobe.h"
#include "config.h"

class Scoring
{
    // This class does static evaluation of chess positions.

    public:
		
    enum { INVALID_SCORE = -Constants::BIG };
#ifdef SMP
    enum { ON_EVALUATION = -Constants::BIG-100 };
#endif

    static void init();
        
    static void cleanup();
		
    // evaluate "board" from the perspective of the side to move.
    static int evalu8( const Board &board );

    // calculate any asymmetrical eval terms
    static int asymm_evalu8( const Board &board, ColorType computerSide);
		
    // return a positional score
    static int positional_score( const Board &board, int alpha, int beta );

    // return a material score
    static int material_score( const Board &board );
		
    // return what the material score will be after move "m"
    // is made. board holds the position before the move is 
    // made. Score is from the perspective of the current
    // side to move.
    static int material_score( const Board &board, Move m );

    static int repetition_draw( const Board &board );

    static int material_draw( const Board &board );
        
    // checks for legal draws plus certain other theoretically
    // draw positions
    static int is_draw(const Board &board);
        
    // Check only for those positions which are not legal draws
    // but are theoretically drawn
    static int is_theoretical_draw(const Board &board);
        
    // Check only for legal draws
    static int is_legal_draw(const Board &board);

    // Turn a score into a formatted string (mate scores are
    // shown like +Mate6).
    static void print_score( int score, char *buf );

    static void print_score_uci( int score, char *buf );

    static void dump_stats();

    static inline int uncatchable(const ColorType sideToMove,
                 const Square pawnpos,
                 const Square kp,
                 const Square oppkp,
                 const ColorType side) {
      return lookupBitbase(kp,pawnpos,oppkp,side,sideToMove);
    }

#ifdef EVAL_STATS
    static void clearStats();
#endif

    // Try to return a score based on bitbases
    static int tryBitbase(const Board &board);

    static int sq_king_test(Square oppkp,Square pawnpos,ColorType side,
      ColorType sideToMove);

protected:
        static int positional_score( const Board &, const ColorType );

};

#endif






























































































































































































































































































































































































































































































































































































































































































































