// Copyright 2005 by Jon Dart. All Rights Reserved.

#ifndef _THREAD_POOL_H
#define _THREAD_POOL_H

#include "types.h"

class Search;
struct NodeInfo;

struct ThreadInfo {
   enum State { Idle, Working, Terminating };
   ThreadInfo();
   virtual ~ThreadInfo();
   State state;
   Search *work;
   NodeInfo *parentNode;
   THREAD thread_id;
#ifdef _WIN32
   HANDLE hEvent1;
#else
   int semid;  // semaphore id
   unsigned count;
#endif
   void start(Search *work);
};

class ThreadPool {
public:

   // create a thread pool with n threads
   ThreadPool(int n);

   virtual ~ThreadPool();

   // obtain an idle thread if possible, returns 
   // non-null if successful
   ThreadInfo *checkOut(NodeInfo *);

   // return a thread to the pool
   void checkIn(ThreadInfo *);

   int activeCount();

private:
   void shutDown();

   LockDefine(poolLock);
   ThreadInfo **data;
   int nthreads;
};

#endif
